/*	==============
 *	AEADescList.cc
 *	==============
 */

#include "AEADebugging.h"

#include <AppleEvents.h>
#include <Errors.h>

#include "AEADescList.hh"

AEADescList::AEADescList()
{
	//InternalCreate(NULL, 0);
}

AEADescList::AEADescList(AEDesc inAEDesc)
: AEADesc(inAEDesc)
{
	if (inAEDesc.descriptorType != 'list')
		ThrowOSErr_(errAEWrongDataType);
}

AEADescList::AEADescList(const void *inFactoringPtr, Size inFactoredSize)
{
	Create(inFactoringPtr, inFactoredSize);
}

AEADescList::~AEADescList()
{
	// AEADesc already calls Dispose().
}

void
AEADescList::Create()
{
	Create(NULL, 0);
}

void
AEADescList::Create(const void *inFactoringPtr, Size inFactoredSize)
{
	OSErr err = ::AECreateList(inFactoringPtr, inFactoredSize, false, &Ref());
	ThrowIfOSErr_(err);
}

long
AEADescList::CountItems() const
{
	long count;
	OSErr err = ::AECountItems(&Ref(), &count);
	ThrowIfOSErr_(err);
	return count;
}

void
AEADescList::Get(long inIndex, DescType inDesiredType, 
	void *outDataPtr, Size inMaximumSize, Size &outActualSize) const
{
	AEKeyword keyword;
	DescType typeCode;
	OSErr err = ::AEGetNthPtr(&Ref(), inIndex, inDesiredType, 
		&keyword, &typeCode, outDataPtr, inMaximumSize, &outActualSize);
	ThrowIfOSErr_(err);
	
	if (inDesiredType != typeCode)
		ThrowOSErr_(errAECoercionFail);
}

void
AEADescList::Get(long inIndex, AEDesc &outAEDesc) const
{
	AEKeyword keyword;
	OSErr err = ::AEGetNthDesc(&Ref(), inIndex, typeWildCard, &keyword, &outAEDesc);
	ThrowIfOSErr_(err);
}

void
AEADescList::Put(long inIndex, DescType inTypeCode, 
	const void *inDataPtr, Size inDataSize)
{
	OSErr err = ::AEPutPtr(&Ref(), inIndex, inTypeCode, inDataPtr, inDataSize);
	ThrowIfOSErr_(err);
}

void
AEADescList::Put(long inIndex, const AEDesc &inAEDesc)
{
	OSErr err = ::AEPutDesc(&Ref(), inIndex, &inAEDesc);
	ThrowIfOSErr_(err);
}

void
AEADescList::Put(AEArrayType inArrayType, const AEArrayDataPointer inArrayPtr, 
	DescType inItemType, Size inItemSize, long inItemCount)
{
	OSErr err = ::AEPutArray(&Ref(), inArrayType, inArrayPtr, inItemType, inItemSize, inItemCount);
	ThrowIfOSErr_(err);
}
