/*	==========
 *	C3DPort.cc
 *	==========
 */

#include <stddef.h>

#include "NGLList.hh"

#include "C3DPort.hh"
#include "C3DObjects.hh"
#include "C3DModel.hh"


double gLines[][2][3] = {
	{{0, 0, 0}, {0, 0, 10}}, 
	{{0, 10, 0}, {0, 10, 10}}, 
	{{10, 0, 0}, {10, 0, 10}}, 
	{{10, 10, 0}, {10, 10, 10}}, 
	{{0, 0, 0}, {0, 10, 0}}, 
	{{0, 0, 10}, {0, 10, 10}}, 
	{{10, 0, 0}, {10, 10, 0}}, 
	{{10, 0, 10}, {10, 10, 10}}, 
	{{0, 0, 0}, {10, 0, 0}}, 
	{{0, 0, 10}, {10, 0, 10}}, 
	{{0, 10, 0}, {10, 10, 0}}, 
	{{0, 10, 10}, {10, 10, 10}}, 
};
short lineCount = 12;

#if 0
void PlotLine(C3DPoint *inPtA, C3DPoint *inPtB)
{
	double x, y, z;
	Point ptScreen, ptMac;
	short width, height;
	short doLine;
	
	width = qd.thePort->portRect.right - qd.thePort->portRect.left;
	height = qd.thePort->portRect.bottom - qd.thePort->portRect.top;
	
	x = inPtA->Cell(0, 0);
	y = inPtA->Cell(0, 1);
	z = inPtA->Cell(0, 2);
	
	if (y > 0) {
		ptScreen.h = mag * dist * x / y;
		ptScreen.v = mag * dist * z / y;
		
		ptMac.h = ptScreen.h + width / 2;
		ptMac.v = ptScreen.v + height / 2;
		
		MoveTo(ptMac.h, ptMac.v);
		PenSize(2, 2);
		Line(0, 0);
		PenSize(1, 1);
	}
	doLine = y > 0;
	
	x = inPtB->Cell(0, 0);
	y = inPtB->Cell(0, 1);
	z = inPtB->Cell(0, 2);
	
	if (y > 0) {
		ptScreen.h = mag * dist * x / y;
		ptScreen.v = mag * dist * z / y;
		
		ptMac.h = ptScreen.h + width / 2;
		ptMac.v = ptScreen.v + height / 2;
		
		doLine ? LineTo(ptMac.h, ptMac.v) : MoveTo(ptMac.h, ptMac.v);
		PenSize(2, 2);
		Line(0, 0);
		PenSize(1, 1);
	}
}

void DoIt()
{
	C3DTransform xform;
	if (1) {
		C3DYaw rotate(turn);
		//C3DYaw *rotate = new C3DYaw(turn);
		xform.MultiplyBy(rotate).MultiplyBy(camera);
		//delete rotate;
	}
	
	for (short i = 0; i < lineCount; i++) {
		double *pt;
		pt = gLines[i][0];
		C3DPoint *pt3dA = new C3DPoint;
		pt3dA->Cell(0, 0) = pt[0];
		pt3dA->Cell(0, 1) = pt[1];
		pt3dA->Cell(0, 2) = pt[2];
		pt3dA->MultiplyBy(xform);
		pt = gLines[i][1];
		C3DPoint *pt3dB = new C3DPoint;
		pt3dB->Cell(0, 0) = pt[0];
		pt3dB->Cell(0, 1) = pt[1];
		pt3dB->Cell(0, 2) = pt[2];
		pt3dB->MultiplyBy(xform);
		PlotLine(pt3dA, pt3dB);
		delete pt3dA;
		delete pt3dB;
	}
}
#endif

C3DPort::C3DPort(C3DModel &inModel)
: mModel(inModel), mCamera(NULL)
{
	//inModel.retain();
}

C3DPort::~C3DPort()
{
	if (mCamera) {
		delete mCamera;  // FIXME:  This is ugly
	}
}

void
C3DPort::SetCamera(C3DCamera *inCamera)
{
	mCamera = inCamera;
}

void
C3DPort::SendCameraCommand(short inCmd)
{
	if (mCamera) {
		double incrMove = 1.0;
		double incrRotate = 2.0 * 3.14159 / 180;
		switch (inCmd) {
			case cmdMoveLeft:
				mCamera->Translate(-incrMove, 0, 0);
				break;
			case cmdMoveRight:
				mCamera->Translate(incrMove, 0, 0);
				break;
			case cmdMoveForward:
				mCamera->Translate(0, incrMove, 0);
				break;
			case cmdMoveBackward:
				mCamera->Translate(0, -incrMove, 0);
				break;
			case cmdMoveUp:
				mCamera->Translate(0, 0, incrMove);
				break;
			case cmdMoveDown:
				mCamera->Translate(0, 0, -incrMove);
				break;
			case cmdYawLeft:
				mCamera->Yaw(-incrRotate);
				break;
			case cmdYawRight:
				mCamera->Yaw(incrRotate);
				break;
			case cmdPitchUp:
				mCamera->Pitch(-incrRotate);
				break;
			case cmdPitchDown:
				mCamera->Pitch(incrRotate);
				break;
			case cmdRollLeft:
				mCamera->Roll(incrRotate);
				break;
			case cmdRollRight:
				mCamera->Roll(-incrRotate);
				break;
			case cmdExpand:
				mCamera->Scale(1.05);
				break;
			case cmdContract:
				mCamera->Scale(1.0 / 1.05);
				break;
			default:
				break;
		}
	}
}


static
void
DeletePoint(C3DPoint *inPoint, void *)
{
	delete inPoint;
}

void
C3DPort::GetCameraFrame()
{
	mPoints.ForEach(&DeletePoint, NULL);
	mPoints.RemoveAll();
	
	long count = mModel.CountDots();
	for (short i = 0; i < count; i++) {
		double x, y, z;
		mModel.GetDot(i, x, y, z);
		C3DPoint *ptWorld = new C3DPoint;
		ptWorld->Cell(0, 0) = x;
		ptWorld->Cell(0, 1) = y;
		ptWorld->Cell(0, 2) = z;
		C3DPoint *ptEye = new C3DPoint;
		C3DPoint *ptPort = new C3DPoint;
		mCamera->WorldToEye(*ptWorld, *ptEye);
		delete ptWorld;
		if (mCamera->EyeToPort(*ptEye, *ptPort)) {
			mPoints.Append(ptPort);
		} else {
			delete ptPort;
		}
		delete ptEye;
	}
}
