/*	===================
 *	OrganExecHandler.cc
 *	===================
 */

#include "AEADebugging.h"

#include <stdio.h>
#include <stdlib.h>

#include <AppleEvents.h>

#include "MoreFilesExtras.h"

#include "POSIX_main.h"

// DeathWarrants
//#include "CDeleteWarrant.hh"
//#include "CFreeWarrant.hh"
//#include "CAEDisposeDescWarrant.hh"
//#include "CExecutioner.hh"

#include "SiesString.hh"

// AEA
#include "AEADescList.hh"
#include "AEADescAppleEvent.hh"

// Pipe Organ
#include "OrganExecHandler.hh"
#include "PipeOrganSuite.h"


template <class Type>
class StackPointer {
public:
	StackPointer(Type *inPtr) : mPtr(inPtr) {}
	virtual ~StackPointer() {delete mPtr;}
protected:
	Type *mPtr;
};

template <class Type>
class StackArray {
public:
	StackArray(Type *inPtr) : mPtr(inPtr) {}
	virtual ~StackArray() {delete [] mPtr;}
protected:
	Type *mPtr;
};

OrganExecHandler::OrganExecHandler()
: AEAHandler(kAEPipeOrganSuite, kAEExec)
{
	MakeNoteToTouchParam(keyDirectObject);
	//MakeNoteToTouchParam(keyCurrentWorkingDir);
}

OrganExecHandler::~OrganExecHandler()
{
}

typedef struct FSDirSpec {
	short reserved;
	short vRefNum;
	long dirID;
} FSDirSpec;

void
OrganExecHandler::HandleEventSelf(const AEADescAppleEvent &inAppleEvent, AEADescAppleEvent &outReply)
{
	OSErr err;
	AEDesc arg;
	AEADescList argList;
	long argCount, index;
	DescType actualType;
	Size actualSize;
	long totalArgLength;
	char **argV, *args, *argn;
	FSSpec argFSS;
	FSDirSpec *dir;
	Boolean isDirectory;
	long returnValue = -1;
	
	// Currently, we're expecting a string or a list of strings.
	inAppleEvent.GetParameter(keyDirectObject, typeWildCard, arg);
	
	if (arg.descriptorType != typeAEList) {
		::AEDisposeDesc(&arg);
		inAppleEvent.GetParameter(keyDirectObject, typeChar, arg);
		SiesString str(arg.dataHandle);
		::AEDisposeDesc(&arg);
		
		returnValue = ExecString(str);
	} else {
		::AEDisposeDesc(&arg);
		
		// Previously, we were expecting a list of strings.  Fix me.
		inAppleEvent.GetParameter(keyDirectObject, typeAEList, argList.Ref());
		
		argCount = argList.CountItems();
		
		// All we're doing here is calculating the total size of all the arg strings.
		totalArgLength = 0;
		for (index = 1; index <= argCount; index++) {
			argList.Get(index, typeChar, NULL, 0, actualSize);
			totalArgLength += actualSize + 1;
		}
		// Allocate a block to hold a pointer to each string, plus one extra pointer.
		argV = new char *[argCount + 1];
		StackArray<char *> argVPtr(argV);
		
		// Zero out the first pointer and skip it (reserving it for later).
		argV[0] = NULL;
		argV++;
		
		// Allocate a block to hold all the strings.
		args = new char [totalArgLength];
		StackArray<char> argsPtr(args);
		
		// Copy the strings from the AEDescList into our structure.
		argn = args;
		for (index = 1; index <= argCount; index++) {
			// Copy the string
			argList.Get(index, typeChar, argn, args + totalArgLength - argn, actualSize);
			// Set the argv pointer
			argV[index - 1] = argn;
			// Terminate with a null
			argn[actualSize] = '\0';
			// Onward!
			argn += actualSize + 1;
		}
		
		// Get the current directory, expecting FSS.
		dir = new FSDirSpec;
		StackPointer<FSDirSpec> dirSpecPtr(dir);
		try {
			inAppleEvent.GetParameter(keyCurrentWorkingDir, typeFSS, actualType, 
				&argFSS, sizeof argFSS, actualSize);
			dir->reserved = 0;
			dir->vRefNum = argFSS.vRefNum;
			::FSpGetDirectoryID(&argFSS, &dir->dirID, &isDirectory);
		} catch (ExceptionCode code) {
			if (code != errAEDescNotFound)
				throw code;
			dir->reserved = 0;
			dir->vRefNum = 0;
			dir->dirID = 0;
		}
		argV[-1] = (char *)dir;
		
		// Attach our flow director to the output console to redirect output.
		
		// Echo the input line to the console.  Use stderr, not stdout.
		if (argCount > 0) {
			fprintf(stderr, "$ ");
			fprintf(stderr, "%s", argV[0]);
			for (index = 1; index < argCount; index++)
				fprintf(stderr, " %s", argV[index]);
			fprintf(stderr, "\n");
		}
		
		// Call the main() function with argv context.
		returnValue = ExecArgList(argCount, argV);
	}
	// Put the return value into the reply.
	outReply.PutParameter(keyDirectObject, typeSInt32, &returnValue, sizeof returnValue);
}

long
OrganExecHandler::ExecString(const SiesString &inCmd)
{
	long result = 0;
	
	return result;
}

long
OrganExecHandler::ExecArgList(long argC, char **argV)
{
	long result = 0;
	
	return result;
}
