/*	===============
 *	GenieConsole.cc
 *	===============
 */

// Standard library
#include <string.h>

// Mac OS Universal Headers
#include <Menus.h>

// Pedestal
#include "PedWindow.hh"
#include "PedPaneTE.hh"
#include "PedPaneSubView.hh"
#include "PedViewSub.hh"
#include "PedViewScrollerTE.hh"
#include "PedViewScroll.hh"
#include "PedScrollbar.hh"

// Genie
#include "GenieConsole.hh"
#include "version.h"

#include "OrganConsole.hh"
#include "execute.hh"


GenieConsole::GenieConsole()
: PedAgent(NULL), mPane(NULL)
{
	mPrompt = "$ ";
	mPromptLen = strlen(mPrompt);
	mCmdStart = 0;
}

GenieConsole::~GenieConsole()
{
	if (mWindow) mWindow->release();
	if (mPane) mPane->release();
}

void
GenieConsole::InitWindow()
{
	if (!mWindow) {
		mWindow = new PedWindow(this);
		
		Rect rect = {18, 0, 18+2*4+11*25+16, 2*4+6*80+16};
		short mbarHeight = ::GetMBarHeight();
		short vMargin = (qd.screenBits.bounds.bottom - rect.bottom) - mbarHeight;
		short hMargin = (qd.screenBits.bounds.right - rect.right);
		::OffsetRect(&rect, hMargin / 2, mbarHeight + vMargin / 3);
		
		// Set the bounds before creating the pane, because things aren't
		// perfect yet and the pane will be sized to the old bounds.
		mWindow->SetBounds(rect);
		mWindow->SetTitle("\pConsole");
		
		//mPane = new PedPaneTE(*mWindow);
		PedPaneSubView *top = new PedPaneSubView(*mWindow);
		mWindow->SetPane(top);
		PedViewScrollerTE *scroller = new PedViewScrollerTE(*top);
		top->SetSubView(scroller);
		scroller->SetScrollbarPresence(kPedScrollVertical, true);
		PedPaneTE *textpane = new PedPaneTE(*scroller);
		scroller->SetPane(textpane);
		mPane = textpane;
	}
}

bool
GenieConsole::OpenWindow()
{
	return PedAgent::OpenWindow();
}

bool
GenieConsole::CloseWindow()
{
	
	return PedAgent::CloseWindow();
}

void
GenieConsole::Prompt()
{
	if (!mPane) throw;
	mCmdStart = mPane->TextLength();
	mPane->AppendChars(mPrompt, mPromptLen);
	mPane->SetSelection(32767, 32767);
}

long
GenieConsole::WriteChars(char *buffer, long n)
{
	if (!mPane) return 0;
	
	if (mWindow && !mWindow->IsInExistence())
		mWindow->Open();
	
	return mPane->AppendChars(buffer, n);
}

void
GenieConsole::ProcessKey(EventRecord &inEvent)
{
	char c = inEvent.message & charCodeMask;
	short code = (inEvent.message & keyCodeMask) >> 8;
	ProcessKey(c, code);
}

void
GenieConsole::ProcessKey(char inChar, short inCode)
{
	char c = inChar;
	
	switch (c) {
		case 0x1c: // left arrow
		case 0x1d: // right arrow
		case 0x1e: // up arrow
		case 0x1f: // down arrow
			mPane->DoKey(c);
			break;
		default:
			break;
	}
}
