/* cp.c */

#include <stdio.h>

#include "MoreFilesExtras.h"
#include "FileCopy.h"
#include "pstring.h"
#include "SiesString.hh"
#include "pathnames.hh"

#define NAM

#ifdef NAM
extern "C" int cp_main(int argc, char *argv[]);
#else
#define cp_main main
#endif

static int TryHFSCopy(
	short srcVol, long srcDir, const unsigned char *srcName,
	short destVol, long destDir, const unsigned char *destName);
	
int cp_main(int argc, char *argv[])
{
	OSErr err;
	FSDirSpec cwdir = {0, 0, 0}, *cwdp;
	FSDirSpec srcDir, destDir;
	SiesString strSrcName, strDestName;
	Str31 srcName, destName;
	int index, fail;
	
	// Current working directory: either passed 'under the table' or set to default.
	cwdp = (argv[-1] != NULL) ? (FSDirSpec *)argv[-1] : (&cwdir);
	
	// Check for sufficient number of args
	if (argc < 3) {
		fprintf(stderr, "cp: missing %s\n", (argc == 1) ? "file arguments" : "destination file");
		return 1;
	}
	
	if (argc > 3) {
		// Copy multiple files.
		
		// Last arg should be the destination directory.
		err = UnixRelativePathname2DirAndName(*cwdp, argv[argc - 1], destDir, strDestName);
		strDestName.GetPStr(destName, 31);
		//Pstrcpy(destname, argv[argc - 1]);
		//err = GetDirectoryID(cwdp->vRefNum, cwdp->dirID, destname, &destDirID, &isDir);
		if (err != noErr || *destName != '\0') {
			fprintf(stderr, "cp: copying multiple files, but last argument (%s) is not a directory.\n", argv[argc - 1]);
			return 1;
		}
		
		// Try to copy each file.  Return whether any errors occurred.
		fail = 0;
		for (index = 1; index < argc - 1; index++) {
			err = UnixRelativePathname2DirAndName(*cwdp, argv[index], srcDir, strSrcName);
			strSrcName.GetPStr(srcName, 31);
			if (err != noErr)
				fail++;
			else
				fail += TryHFSCopy(srcDir.vRefNum, srcDir.dirID, srcName, destDir.vRefNum, destDir.dirID, srcName);
		}
		return (fail == 0) ? 0 : 1;
	} else {
		// Copy single file.
		
		err = UnixRelativePathname2DirAndName(*cwdp, argv[1], srcDir, strSrcName);
		strSrcName.GetPStr(srcName, 31);
		if (err != noErr) {
			char name[32];
			strPcpy(name, srcName);
			fprintf(stderr, "cp: %s: %s %d\n", name, "pathname conversion error", err);
			return 1;
		}
//		Pstrcpy(destname, argv[2]);
		// Scope out the destination.
		err = UnixRelativePathname2DirAndName(*cwdp, argv[2], destDir, strDestName);
		strDestName.GetPStr(destName, 31);
//		err = GetDirectoryID(cwdp->vRefNum, cwdp->dirID, destname, &destDirID, &isDir);
		// Error may indicate nonexistent destination item -- try a lateral copy.
		// If destination is an existing file, try a lateral copy (overwriting it).
		// If dest is a directory, try a downward copy into it, using source filename.
		if (err != noErr || *destName != '\0') {
			// Copy this -> that
			fail = TryHFSCopy(srcDir.vRefNum, srcDir.dirID, srcName, destDir.vRefNum, destDir.dirID, destName);
		} else {
			// Copy this -> that/this
			fail = TryHFSCopy(srcDir.vRefNum, srcDir.dirID, srcName, destDir.vRefNum, destDir.dirID, srcName);
		}
		
		return fail;
	}
	
	return 0;
}

int TryHFSCopy(
	short srcVol, long srcDir, const unsigned char *srcName,
	short destVol, long destDir, const unsigned char *destName)
{
	OSErr err;
	//Str255 srcPStr, destPStr;
	short vol1, vol2;
	long dir1, dir2;
	Str255 name1, name2;
	Boolean isDir;
	
	//Pstrcpy(srcPStr, srcName);
	err = GetObjectLocation(srcVol, srcDir, srcName, &vol1, &dir1, name1, &isDir);
	
	char srcname[32], destname[32];
	strPcpy(srcname, srcName);
	strPcpy(destname, destName);
	
	if (err != noErr) {
		// Source item does not exist.
		fprintf(stderr, "cp: %s: %s\n", srcname, "No such file or directory");
		return 1;
	} else if (isDir) {
		// Source item is a directory.
		fprintf(stderr, "cp: %s: %s\n", srcname, "omitting directory");
		return 1;
	}
	//Pstrcpy(destPStr, destName);
	err = GetObjectLocation(destVol, destDir, destName, &vol2, &dir2, name2, &isDir);
	if (err == noErr) {
		// Dest file exists
		if (vol1 == vol2 && dir1 == dir2 && Pstreq(name1, name2)) {
			// Source and destination are the same file!
			fprintf(stderr, "cp: `%s' and `%s' are the same file\n", srcname, destname);
			return 1;
		} else {
			// Distinct dest file exists.  Delete it.
			err = HDelete(vol2, dir2, name2);
			if (err != noErr) {
				fprintf(stderr, "cp: %s: %s %d\n", destname, "HDelete error", err);
			}
		}
	}
	err = FileCopy(vol1, dir1, name1, vol2, dir2, NULL, name2, NULL, 0, true);
	if (err != noErr) {
		fprintf(stderr, "cp: %s: %s %d\n", srcname, "FileCopy error", err);
		return 1;
	}
	return 0;
}
