/* mv.c */

#include <stdio.h>

#include "MoreFilesExtras.h"
#include "pstring.h"
#include "SiesString.hh"
#include "pathnames.hh"

#define NAM

#ifdef NAM
extern "C" int mv_main(int argc, char *argv[]);
#else
#define mv_main main
#endif

static int TryHFSMoveRename(
	short srcVol, long srcDir, const unsigned char *srcName,
	long destDir, const unsigned char *destName);
	
int mv_main(int argc, char *argv[])
{
	OSErr err;
	FSDirSpec cwdir = {0, 0, 0}, *cwdp;
	FSDirSpec srcDir, destDir;
	Str31 srcName, destName;
	SiesString strSrcName, strDestName;
	//Str255 destname;
	//long destDirID;
	Boolean isDir;
	int index, fail;
	
	cwdp = (argv[-1] != NULL) ? (FSDirSpec *)argv[-1] : (&cwdir);
	
	// Check for sufficient number of args
	if (argc < 3) {
		fprintf(stderr, "mv: missing %s\n", (argc == 1) ? "file arguments" : "destination file");
		return 1;
	}
	
	if (argc > 3) {
		// Move multiple files.
		
		err = UnixRelativePathname2DirAndName(*cwdp, argv[argc - 1], destDir, strDestName);
		strDestName.GetPStr(destName, 31);
		//Pstrcpy(destname, argv[argc - 1]);
		//err = GetDirectoryID(cwdp->vRefNum, cwdp->dirID, destname, &destDirID, &isDir);
		if (err != noErr || strDestName.Length() > 0) {
			fprintf(stderr, "mv: moving multiple files, but last argument (%s) is not a directory.\n", argv[argc - 1]);
			return 1;
		}
		
		fail = 0;
		for (index = 1; index < argc - 1; index++) {
			err = UnixRelativePathname2DirAndName(*cwdp, argv[index], srcDir, strSrcName);
			strSrcName.GetPStr(srcName, 31);
			if (err != noErr)
				fail++;
			else
				fail += TryHFSMoveRename(srcDir.vRefNum, srcDir.dirID, srcName, destDir.dirID, NULL);
		}
		return (fail == 0) ? 0 : 1;
	} else {
		// Move single file or directory.
		
		err = UnixRelativePathname2DirAndName(*cwdp, argv[1], srcDir, strSrcName);
		strSrcName.GetPStr(srcName, 31);
		if (err != noErr) {
			char name[32];
			strPcpy(name, srcName);
			fprintf(stderr, "mv: %s: %s %d\n", name, "pathname conversion error", err);
			return 1;
		}
//		Pstrcpy(destname, argv[2]);
		err = UnixRelativePathname2DirAndName(*cwdp, argv[2], destDir, strDestName);
		strDestName.GetPStr(destName, 31);
//		err = GetDirectoryID(cwdp->vRefNum, cwdp->dirID, destname, &destDirID, &isDir);
		if (err != noErr || strDestName.Length() > 0) {
			// Move this -> that
			fail = TryHFSMoveRename(srcDir.vRefNum, srcDir.dirID, srcName, destDir.dirID, destName);
		} else {
			// Move this -> that/this
			fail = TryHFSMoveRename(srcDir.vRefNum, srcDir.dirID, srcName, destDir.dirID, NULL);
		}
		
		return fail;
	}
	
	return 0;
}

/* This code may not be complete */
/*
	srcVol, srcDir, and srcName specify a source item.
	destDir and destName specify a destination directory on the same volume.
	Our mission:
		If no item named destName exists in destDir, move/rename srcName to destDir/destName.
		Otherwise, if srcName in srcDir and destName in destDir are both files,
			then delete destName and move/rename srcName, as above.
		Otherwise, report an error and return.
*/

int TryHFSMoveRename(
	short srcVol, 
	long srcDir, 
	const unsigned char *srcName, 
	long destDir, 
	const unsigned char *destName)
{
	OSErr err;
	Str255 srcPStr, destPStr;
	short vol1, vol2;
	long dir1, dir2;
	Str255 name1, name2;
	Boolean renaming, isDir1, isDir2;
	
	if (destName == NULL) {
		destName = srcName;
		renaming = false;
	} else
		renaming = true;
		
	char src[32], dest[32];
	strPcpy(src, srcName);
	strPcpy(dest, destName);
	
	//Pstrcpy(srcPStr, srcName);
	// Check out source item.
	err = GetObjectLocation(srcVol, srcDir, srcName, &vol1, &dir1, name1, &isDir1);
	if (err != noErr) {
		fprintf(stderr, "mv: %s: %s\n", src, "No such file or directory");
		return 1;
	}
	
	err = GetObjectLocation(srcVol, destDir, renaming ? destName : name1, 
			&vol2, &dir2, name2, &isDir2);
	if (err == noErr) {
		// Dest file exists
		if (vol1 == vol2 && dir1 == dir2 && Pstreq(name1, name2)) {
			// Source and destination are the same file!
			fprintf(stderr, "mv: `%s' and `%s' are the same file\n", src, dest);
			return 1;
		} else {
			// Distinct dest item exists.  Delete it if both src and dest are files.
			if (isDir2) {
				fprintf(stderr, "mv: %s: %s\n", dest, "Can't replace a directory");
				return 1;
			} else if (isDir1) {
				fprintf(stderr, "mv: %s: %s\n", dest, "Can't replace a file with a directory");
				return 1;
			}
			err = HDelete(vol2, dir2, name2);
			if (err != noErr) {
				fprintf(stderr, "mv: %s: %s %d\n", dest, "HDelete error", err);
			}
		}
	}
	
	// If dir1 and dir2 are the same, use Rename.
	if (dir1 == dir2)
		err = HRename(vol1, dir1, name1, name2);
	else
		err = HMoveRenameCompat(vol1, dir1, name1, dir2, NULL, (renaming ? name2 : NULL));
	if (err != noErr) {
		fprintf(stderr, "mv: %s: %s %d\n", src, "Rename or HMoveRenameCompat error", err);
		return 1;
	}
	return 0;
}
