/*	==========
 *	PedPane.cc
 *	==========
 */

#include "PedestalDebugging.h"

#include "PedPane.hh"
#include "PedView.hh"


PedPane::PedPane(PedView &inSuperView)
: mSuperView(inSuperView)
{
	const Rect emptyRect = {0, 0, 0, 0};
	mBounds = mAperture = emptyRect;
	inSuperView.GetFrame(mBounds);
	inSuperView.retain();
	inSuperView.Focus();
}

PedPane::~PedPane()
{
	Close();
	mSuperView.release();
}

void
PedPane::Dispose()
{
	Close();
}

void
PedPane::GetBounds(Rect &outBounds)
{
	outBounds = mBounds;
}

void
PedPane::SetBounds(const Rect &inBounds)
{
	mBounds = inBounds;
}

void
PedPane::GetAperture(Rect &outAperture)
{
	outAperture = mAperture;
}

void
PedPane::SetAperture(const Rect &inAperture)
{
	mAperture = inAperture;
}

bool
PedPane::PointInBounds(Point inPoint)
{
	// Assume inPoint is in view/frame coordinates
	Rect bounds;
	GetBounds(bounds);
	return ::PtInRect(inPoint, &bounds);
}

bool
PedPane::PointInAperture(Point inPoint)
{
	// Assume inPoint is in view/frame coordinates
	Rect aperture;
	GetAperture(aperture);
	return ::PtInRect(inPoint, &aperture);
}


void
PedPane::Open()
{
}

void
PedPane::Close()
{
}

void
PedPane::Activate()
{
}

void
PedPane::Deactivate()
{
}

void
PedPane::Resize(short inWidth, short inHeight)
{
	mBounds.right = mBounds.left + inWidth;
	mBounds.bottom = mBounds.top + inHeight;
	// FIXME:  This may not be correct
	mAperture.right = mAperture.left + inWidth;
	mAperture.bottom = mAperture.top + inHeight;
}

void
PedPane::DrawContent()
{
	Draw();
}

void
PedPane::Draw()
{
}

void
PedPane::DispatchNullEvent(EventRecord &inEvent)
{
	
}

void
PedPane::DispatchClickEvent(EventRecord &inEvent)
{
}

void
PedPane::DispatchKey(EventRecord &inEvent)
{
}

void
PedPane::Cut()
{
}

void
PedPane::Copy()
{
}

void
PedPane::Paste()
{
}

void
PedPane::Clear()
{
}
